@@ -1,12 +1,12 @@
 """Bridge: Init"""
-from systembridge.objects.open.payload import OpenPayload
+from __future__ import annotations
+from systembridge.objects.events import Event, EventBase
 from aiohttp import ClientResponse
-from typing import Any, List, Optional
+from typing import Any, List
 
 from .client import BridgeClient
+from .client_websocket import BridgeClientWebSocket
 from .objects.audio import Audio
-from .objects.audio.delete import AudioDeleteResponse
-from .objects.audio.post import AudioPostPayload, AudioPostResponse
 from .objects.audio.put import AudioPutPayload, AudioPutResponse
 from .objects.base import BridgeBase
 from .objects.battery import Battery
@@ -18,15 +18,17 @@ from .objects.display import Display
 from .objects.display.put import DisplayPutPayload
 from .objects.filesystem import Filesystem
 from .objects.graphics import Graphics
-from .objects.information import Information
+from .objects.media import Media
+from .objects.media.delete import MediaDeleteResponse
+from .objects.media.post import MediaPostPayload, MediaPostResponse
+from .objects.media.put import MediaPutPayload, MediaPutResponse
 from .objects.memory import Memory
 from .objects.network import Network
+from .objects.open.payload import OpenPayload
 from .objects.os import Os
 from .objects.processes import Processes
+from .objects.settings import Settings
 from .objects.system import System
-from .objects.video.delete import VideoDeleteResponse
-from .objects.video.post import VideoPostPayload, VideoPostResponse
-from .objects.video.put import VideoPutResponse
 
 BASE_HEADERS = {"Accept": "application/json"}
 
@@ -39,18 +41,21 @@ class Bridge(BridgeBase):
         self._client = client
         self._base_url = base_url
         self._api_key = api_key
-        self._audio: Audio = []
+        self._audio: Audio = {}
         self._battery: Battery = {}
         self._bluetooth: Bluetooth = {}
         self._cpu: Cpu = {}
         self._display: Display = {}
         self._filesystem: Filesystem = {}
         self._graphics: Graphics = {}
-        self._information: Information = {}
+        self._media: Media = {}
         self._memory: Memory = {}
+        self._network: Network = {}
         self._os: Os = {}
         self._processes: Processes = {}
+        self._settings: List[Settings] = []
         self._system: System = {}
+        self._websocket_client: BridgeClientWebSocket = None
 
     @property
     def audio(self) -> Audio:
@@ -81,8 +86,8 @@ class Bridge(BridgeBase):
         return self._graphics
 
     @property
-    def information(self) -> Information:
-        return self._information
+    def media(self) -> Media:
+        return self._media
 
     @property
     def memory(self) -> Memory:
@@ -100,11 +105,15 @@ class Bridge(BridgeBase):
     def processes(self) -> Processes:
         return self._processes
 
+    @property
+    def settings(self) -> List[Settings]:
+        return self._settings
+
     @property
     def system(self) -> System:
         return self._system
 
-    async def async_post(self, path: str, payload: Optional[Any]) -> Any:
+    async def async_post(self, path: str, payload: Any | None) -> Any:
         """Generic Delete"""
         response: ClientResponse = await self._client.post(
             f"{self._base_url}{path}",
@@ -119,7 +128,9 @@ class Bridge(BridgeBase):
             f"{self._base_url}{path}",
             headers={**BASE_HEADERS, "api-key": self._api_key},
         )
-        return await response.json()
+        if "application/json" in response.headers.get("Content-Type", ""):
+            return await response.json()
+        return await response.text()
 
     async def async_post(self, path: str, payload: Any) -> Any:
         """Generic post"""
@@ -128,37 +139,31 @@ class Bridge(BridgeBase):
             headers={**BASE_HEADERS, "api-key": self._api_key},
             json=payload,
         )
-        return await response.json()
+        if "application/json" in response.headers.get("Content-Type", ""):
+            return await response.json()
+        return await response.text()
 
     async def async_put(self, path: str, payload: Any) -> Any:
         """Generic put"""
-        response: ClientResponse = await self._client.post(
+        response: ClientResponse = await self._client.put(
             f"{self._base_url}{path}",
             headers={**BASE_HEADERS, "api-key": self._api_key},
             json=payload,
         )
-        return await response.json()
-
-    async def async_stop_audio_player(self) -> AudioDeleteResponse:
-        """Stop audio player"""
-        return AudioDeleteResponse(await self.async_delete("/audio"))
-
-    async def async_create_audio_player(
-        self, payload: AudioPostPayload
-    ) -> AudioPostResponse:
-        """Create audio player"""
-        return AudioPostResponse(await self.async_post("/audio", payload))
+        if "application/json" in response.headers.get("Content-Type", ""):
+            return await response.json()
+        return await response.text()
 
     async def async_get_audio(self) -> Audio:
         """Get audio information"""
-        self._audio = [Audio(audio) for audio in await self.async_get("/audio") or []]
+        self._audio = Audio(await self.async_get("/audio"))
         return self._audio
 
     async def async_update_audio(
         self, id: str, payload: AudioPutPayload
     ) -> AudioPutResponse:
         """Update audio"""
-        return AudioPutResponse(await self.async_post(f"/audio/{id}", payload))
+        return AudioPutResponse(await self.async_put(f"/audio/{id}", payload))
 
     async def async_get_battery(self) -> Battery:
         """Get battery information"""
@@ -188,7 +193,7 @@ class Bridge(BridgeBase):
         self, id: str, payload: DisplayPutPayload
     ) -> Display:
         """Update display"""
-        return AudioPutResponse(await self.async_post(f"/display/{id}", payload))
+        return AudioPutResponse(await self.async_put(f"/display/{id}", payload))
 
     async def async_get_filesystem(self) -> Filesystem:
         """Get filesystem information"""
@@ -200,10 +205,13 @@ class Bridge(BridgeBase):
         self._graphics = Graphics(await self.async_get("/graphics"))
         return self._graphics
 
-    async def async_get_information(self) -> Information:
-        """Get information"""
-        self._information = Information(await self.async_get("/information"))
-        return self._information
+    async def async_get_media(self) -> Media | None:
+        """Get media information"""
+        media = await self.async_get("/media")
+        if media is None:
+            return None
+        self._media = Media(media)
+        return self._media
 
     async def async_get_memory(self) -> Memory:
         """Get memory information"""
@@ -229,21 +237,56 @@ class Bridge(BridgeBase):
         self._processes = Processes(await self.async_get("/processes"))
         return self._processes
 
+    async def async_get_settings(self) -> List[Settings]:
+        """Get settings"""
+        settings = await self.async_get("/settings")
+        self._settings = [Settings(setting) for setting in settings]
+        return self._settings
+
+    async def async_get_setting(self, section: str, key: str) -> Settings:
+        """Get setting"""
+        return Settings(await self.async_get(f"/settings/{section}/{key}"))
+
+    async def async_update_setting(
+        self, section: str, key: str, payload: MediaPutPayload
+    ) -> Settings:
+        """Update setting"""
+        return Settings(await self.async_put(f"/settings/{section}/{key}", payload))
+
     async def async_get_system(self) -> System:
         """Get system information"""
         self._system = System(await self.async_get("/system"))
         return self._system
 
-    async def async_stop_video_player(self) -> VideoDeleteResponse:
-        """Stop video player"""
-        return VideoDeleteResponse(await self.async_delete("/video"))
+    async def async_stop_media_player(self) -> MediaDeleteResponse:
+        """Stop media player"""
+        return MediaDeleteResponse(await self.async_delete("/media"))
 
-    async def async_create_video_player(
-        self, payload: VideoPostPayload
-    ) -> VideoPostResponse:
-        """Create video player"""
-        return VideoPostResponse(await self.async_post("/video", payload))
+    async def async_create_media_player(
+        self, payload: MediaPostPayload
+    ) -> MediaPostResponse:
+        """Create media player"""
+        return MediaPostResponse(await self.async_post("/media", payload))
 
-    async def async_update_video(self, id: str) -> VideoPutResponse:
-        """Update video"""
-        return VideoPutResponse(await self.async_post(f"/video/{id}"))
+    async def async_update_media(
+        self, id: str, payload: MediaPutPayload
+    ) -> MediaPutResponse:
+        """Update media player"""
+        return MediaPutResponse(await self.async_put(f"/media/{id}", payload))
+
+    async def async_connect_websocket(self, host: str, port: int) -> None:
+        self._websocket_client: BridgeClientWebSocket = BridgeClientWebSocket(
+            self._api_key
+        )
+        await self._websocket_client.connect(f"ws://{host}:{port}")
+        await self._websocket_client.authenticate()
+
+    async def async_send_event(self, event: str, data: Event) -> None:
+        await self._websocket_client.send_event(event, data)
+
+    async def listen_for_events(self, callback: function) -> None:
+        async def handle_message(message: EventBase) -> None:
+            if "data" in message and type(message["data"]) is dict:
+                await callback(message["data"])
+
+        await self._websocket_client.listen_for_messages(handle_message)
